/*
 * sjm4306
 * IR Remoteduino example program: Remote sleeps while not in use
 * When a button is pressed the remote wakes, blasts IR code and goes back to sleep
 */

//CHECKED with internal 8MHz clk and seems to run correctly (make sure 3.3V 8MHz is selected in tools>processor menu above before compiling)
#include <IRremote.h>
#include <avr/wdt.h>
//ir led on pin D3
#define LED 13
#define pwr 2
#define play 4
#define pause 5
#define ffw 6
#define rew 7
#define volup 8
#define voldn 9
#define stp 10
#define mode 12

#define one 19
#define two 18
#define three 17
#define four 16
#define five 15
#define six 14

IRsend irsend;

unsigned long tmp = 0;
const unsigned int bits = 12;
unsigned long power_cmd = 0b111101010001;
unsigned long play_cmd =   0b010011010001;  //works
unsigned long pause_cmd =  0b100111010001;  //works
unsigned long stop_cmd =   0b000111010001;  //works
unsigned long ffw_cmd =    0b100011010001;  //works
unsigned long rew_cmd =    0b000011010001;  //works
unsigned long volup_cmd = 0b010010010001;
unsigned long voldn_cmd = 0b110010010001;
unsigned long mode_cmd = 0b011010010001;

unsigned long one_cmd = 0b000000010001;
unsigned long two_cmd = 0b000000010001;
unsigned long three_cmd = 0b000000010001;
unsigned long four_cmd = 0b000000010001;
unsigned long five_cmd = 0b000000010001;
unsigned long six_cmd = 0b000000010001;

//0b000000010001 goto 1st track
//0b000001010001 goto 10th track
//0b111000010001 +10 track
//0b011010010001 eject disc

ISR(PCINT0_vect){
   PCIFR&=0b11111110;
}

ISR(PCINT1_vect){
   PCIFR&=0b11111101;
}

ISR(PCINT2_vect){
   PCIFR&=0b11111011;
}

void config_slp(void){
  wdt_disable();  //disable watchdog
  ADCSRA &= ~(1<<7);   //ADEN=0, turn off adc
  PCMSK0=0b00010111;
  PCMSK1=0b00111111; //turn on individual pin change interrupts for all button inputs
  PCMSK2=0b11110100;
  PCIFR=0x00; //clear all interrupt flags
  PCICR=0b00000111; //turn on interrupts for pin changes on three banks of inputs
  SMCR=0b00000101;  //enable sleeping, power down mode
  PRR=0b10000111; //power down individual peripherals to lower current consumption, check if shutting off timers will break irsend!!!!!
}

void setup(){
  pinMode(LED,OUTPUT);
  pinMode(pwr,INPUT_PULLUP);
  pinMode(play,INPUT_PULLUP);
  pinMode(pause,INPUT_PULLUP);
  pinMode(stp,INPUT_PULLUP);
  pinMode(ffw,INPUT_PULLUP);
  pinMode(rew,INPUT_PULLUP);
  pinMode(volup,INPUT_PULLUP);
  pinMode(voldn,INPUT_PULLUP);

  pinMode(mode,INPUT_PULLUP);
  pinMode(one,INPUT_PULLUP);
  pinMode(two,INPUT_PULLUP);
  pinMode(three,INPUT_PULLUP);
  pinMode(four,INPUT_PULLUP);
  pinMode(five,INPUT_PULLUP);
  pinMode(six,INPUT_PULLUP);

  pinMode(0,INPUT);
  pinMode(1,INPUT);

  config_slp();
}

void loop() { //MUST set BOD to <3V so this can all run off a single CR2032 (write 0x05 to extended fuse for 2.7V BOD, 0x07 to disable BOD)
  if(digitalRead(pwr)==0){tmp=power_cmd;  //BUG, if button is held, sends additional press on button release!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  }else if(digitalRead(play)==0){tmp=play_cmd;
  }else if(digitalRead(pause)==0){tmp=pause_cmd;
  }else if(digitalRead(stp)==0){tmp=stop_cmd;
  }else if(digitalRead(ffw)==0){tmp=ffw_cmd;  //modify to advance through track when button held
  }else if(digitalRead(rew)==0){tmp=rew_cmd;
  }else if(digitalRead(volup)==0){tmp=volup_cmd;
  }else if(digitalRead(voldn)==0){tmp=voldn_cmd;
  }else if(digitalRead(mode)==0){tmp=mode_cmd;
  }else if(digitalRead(one)==0){tmp=one_cmd;
  }else if(digitalRead(two)==0){tmp=two_cmd;
  }else if(digitalRead(three)==0){tmp=three_cmd;
  }else if(digitalRead(four)==0){tmp=four_cmd;
  }else if(digitalRead(five)==0){tmp=five_cmd;
  }else if(digitalRead(six)==0){tmp=six_cmd;
  }else{tmp=0;}

  if(tmp!=0){
  	for (int i = 0; i < 3; i++) {
      digitalWrite(LED,HIGH);
  		irsend.sendSony(tmp, bits); //send ir command/address
  		delay(40);
  	}
    digitalWrite(LED,LOW);
  	delay(250); //delay between each signal burst
    tmp=0;
	}

  asm("sei"); //enable global interrupts
  MCUCR |= (3<<5); //set both BODS and BODSE at the same time
  MCUCR = (MCUCR & ~(1<<5))|(1<<6); //then set BODS bit and clear BODSE bit at the same time
  asm("sleep"); //go to sleep, check power consumption in sleep!!!!
}
